// Dictionary.cpp: implementation of the Dictionary class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "project2.h"
#include "Dictionary.h"
#include <iostream>
#include <fstream>
#include <strstream>
#include <stdlib.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

using namespace std;

int compare( const void *arg1, const void *arg2 );

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
Dictionary::~Dictionary()
{
	// During destruction, remove all the dictionary CString entries
	for (int i=0; i<numWords; i++)
	{
		free(words[i]);
	}
}

Dictionary::Dictionary(CString value)
{
	filename=value;
	numWords=0;
}

Dictionary::Dictionary()
{
	numWords=0;
}

void Dictionary::loadDictionary()
{
	// Maximum size of a dictionary word is 1000 characters
	const int MAXWORDSIZE = 1000;

	ifstream is;

	// Open the dictionary file and exit if open fails
	is.open(filename);
	if (is.fail()) {
		cout << "ERROR: Unable to open -> " << filename << endl;
		exit(-1);
	}

	// Read each word from dictionary file into words[] array
	while (!is.eof()) {
		char raw[MAXWORDSIZE];
		is >> raw;
		words[numWords++]= new CString(raw);

		// Make sure we don't read in too many words
		if (numWords > MAXNUMWORDS) {
			cout << "Dictionary::loadDictionary: ERROR!  Number of dictionary words has exceeded maximum.  Exiting." << endl;
			exit(-1);
		}
	}


	// Done with dictionary file... close it.
	is.close();
}


void Dictionary::display()
{
	cout << "**** Displaying Dictionary ****" << endl;
	for (int i=0; i<numWords; i++)
	{
		cout << (LPCTSTR) (*words[i]) << endl;
	}
}

bool Dictionary::find(CString word)
{
	// Start at the beginning of the words[] array and scan
	// it for the word passed in.  If we find the word, return TRUE.
	// If we don't find the word, we'll get to the end of the array,
	// exit the loop and return false.
	for (int i=0; i<numWords; i++) {
		if (word == *words[i]) {
			return true;
		}
	}

	return false;
}
