// project2.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include <cmath>
#include <ctime>
#include <strstream>
#include "Dictionary.h"
#include "project2.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// The one and only application object

CWinApp theApp;

using namespace std;

char generateRandomCharacter();
// Precondition:	none
// Postcondition:	returns a random character a-z or blank

void checkForWords(CString theString);
// Precondition:	theString is a string which contains groups characters separated by blanks
// Postcondition:	outputs to stdout any words located in theString that are also in the dictionary
//					a word is defined as a group of characters separated by a blank on either side

void computeMonkeyString(CString& monkeyString);
// Precondition:	monkeyString is an empty string
// Postcondition:	monkeyString contains groups of random characters (a-z) separated by blanks

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// TODO: change error code to suit your needs
		cerr << _T("Fatal Error: MFC initialization failed") << endl;
		nRetCode = 1;
	}
	else
	{
		CString monkeyString;

		cout << "MonkeyTyping..." << endl;
		computeMonkeyString(monkeyString);

		cout << "Checking for words... " << endl;
		checkForWords(monkeyString);

		cout << "Done... type a character and press enter to exit -> ";
		char atEnd;
		cin >> atEnd;
	}

	return nRetCode;
}

char generateRandomCharacter()
{
	static boolean initialized=false;

	// Initialize rand() function with current time
	if (!initialized) {
		srand(time(0));
		initialized=true;
	}

	int raw = rand() % 27;

	// There are 26 letters in the alphabet.  The 27th letter
	// is a special case... it's a blank.
	if (raw==26) {
		return ' ';
	}

	return (char) (raw + 'a');
}


void checkForWords(CString theString)
{
	bool done = false;
	istrstream is((LPCTSTR) theString);
	char raw[1000];


	// Load the dictionary
	cout << "Loading dictionary... " << endl;
	Dictionary d("dictionary.txt");
	d.loadDictionary();

	// Check each word in theString
	cout << "Checking words..." << endl;
	while (!is.fail())
	{
		is >> raw;
	
		// POSSIBLE MEMORY PROBLEM:  It's possible that the monkey typed
		// a word that has more than 1000 characters, although it is EXTREMELY
		// unlikely... how unlikely?  
		//
		//							(26/27)^N 
		//
		// When N = 1000, it's 4 x 10^-17
		if (strlen(raw) > 999) {
			cout << "Error!  Word too long... skipping" << endl;
		}

		// Convert from raw string to CString
		CString word = raw;

		// Lookup the word in the dictionary
		if (d.find(word)) {
			cout << (LPCTSTR) word << endl;
		}
	} 
}

void computeMonkeyString(CString& monkeyString)
{
	// How many characters does a monkey type in a MONTH?
	//		6 chars per word
	//		50 words per minute
	//		60 minutes per hour
	//		24 hours per day
	//		30 days per month
	const long MONKEYTYPES = 6*50*60*24*30;
	
	// Feedback every FEEDBACK characters
	const long FEEDBACK = 1000;

	for (int i=0; i<MONKEYTYPES; i++) {
		monkeyString += generateRandomCharacter();
		
		// Provide feedback every FEEDBACK characters to let 
		// the user know there's processing going on
		if (i % (1000) == 0) {
			cout << "Character#: " << i << endl;
		}
	}

}

